"""Code generated by __main__"""

from dataclasses import dataclass, field

from sigtech.api.data.utils import BaseRule


@dataclass
class FillCategoricalMissingValue(BaseRule):
    """
    DaVe transform that replaces missing values in the `self.column` column
    (e.g. CreditRating) with the most common value found when aggregating rows
    by the `self.category_column` column (e.g. CompanyName). A threshold
    must be reached for the most common value to be used.

    Use Case:
        Null value is found for a bond credit rating issued by Apple. If at least 90%
        of bonds issued by Apple are a particular value,suggest this value.

    Example:
        # Given
            BondId  Coupon  Tenor  CreditRating  CompanyName
            A001    3.5     5      A+           APPL
            A002    4.2     7      A            MSFT
            A003    2.8     3      A+           APPL
            A004    3.9     10     null         APPL
            A005    2.5     2      null         APPL
            A006    4.1     5      A            NFLX
            A007    3.3     4      A+           APPL
            A008    4.7     8      null         MSFT
            A009    2.1     6      B            MSFT
            A010    3.8     9      A+           APPL
            A011    4.5     7      A            NFLX
            A012    3.2     4      A+           APPL
            A013    4.2     5      A            NFLX
            A014    2.9     6      B            MSFT
            A015    4.0     8      A+           APPL
            A016    3.1     3      null         NFLX
            A017    4.8     5      A            MSFT
            A018    2.4     7      B+           APPL
            A019    4.3     9      B            NFLX
            A020    3.6     6      B            APPL

        # When a transform is configured as follows:
            {
                "type": "FillCategoricalMissingValue",
                "properties": {
                    "column": "CreditRating",
                    "category_column": "CompanyName",
                    "threshold": 70.0,
                }
            }

        # Then we expect the following to happen:
                CompanyName CreditRating  Count  Coverage
                     APPL           A+      6      75.0     <--- 75% > 70%: fill CreditRating nulls with A+ for AAPL
                     MSFT            A      2      50.0     <--- 50% < 70%: keep CreditRating nulls for MSFT
                     NFLX            A      3      75.0     <--- 75% > 70%: fill CreditRating nulls with A for NFLX

    :param column (str): The column in which to fill missing values.
        - example: CreditRating
    :param category_column (str): The column to aggregate common values by.
        - example: CompanyName
    :param threshold (float): Threshold for percentage of values in a column that must be the same to fill missing values.
        - min: 0.0
        - max: 100.0
        - example: 90.0

    docs: https://sigtech.gitbook.io/dave/rules#fill-categorical-missing-value
    """  # noqa: E501

    column: str = field(
        metadata={
            "description": "The column in which to fill missing values.",
            "example": "CreditRating",
            "type": "str",
        }
    )

    category_column: str = field(
        metadata={
            "description": "The column to aggregate common values by.",
            "example": "CompanyName",
            "type": "str",
        }
    )

    threshold: float = field(
        metadata={
            "description": (
                "Threshold for percentage of values in a column that must be the same"
                " to fill missing values."
            ),
            "min": 0.0,
            "max": 100.0,
            "example": 90.0,
            "type": "float",
        }
    )
